/*
* ClmChrono 1.1.1 by Franois CREVOLA - francois@crevola.org
*  License : GPL V2.0 (excluding the ClmChrono.png and ClmChronoSplash.png files which are (c) CourirLeMonde.org)
*/

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.*;
import java.util.*;

// classe principale
public class ClmChrono extends MIDlet implements CommandListener {

  public Display _display; // Display est indispensable  l'affichage
  private ClmEngine _moteur; 
  private ClmTimer _clmTimer; // objet qui excute une mthode  intervalle rgulier
  private Timer _timer; // gestion du temps
  private boolean firstStartDone;

  public Command startCommand, stopCommand, resetCommand, exportCommand, sendCommand;
  public Command optionCommand, lapCommand, backFromExportCommand;
  public Command okCommand;
  private TextField fullDistanceField,lapDistanceField,pseudoField,exportServerField;
  private TextField  titreField;
  private StringItem statusStringItem;
  private ChoiceGroup choixTaille;
  private Form  optionForm;
        
  // constructeur, appel au dmarrage
  public ClmChrono() {
  
    // initialisations
    _moteur = new ClmEngine();
    _clmTimer = new ClmTimer(_moteur);
    _timer = new Timer();
    _display = Display.getDisplay(this);

    firstStartDone = false;

    startCommand = new Command("Start", 1, 1);
    stopCommand = new Command("Stop", 1, 1);
    resetCommand = new Command("Reset", 1, 1);
    backFromExportCommand = new Command("Sortir", 1, 1);
    sendCommand = new Command("Envoyer", 7, 1);
    exportCommand = new Command("Export", 7, 1);
    optionCommand = new Command("Param", 7, 1);
    lapCommand = new Command("Lap", 7, 1);

    _moteur.setParentClass(this);
    _moteur.initChronoCommand();
    _moteur.setCommandListener(this);

    okCommand = new Command("OK", 7, 1);
    lapDistanceField = new TextField("Lap (m)", null, 5, TextField.NUMERIC);
    fullDistanceField = new TextField("Distance (m)", null, 5, TextField.NUMERIC);
    pseudoField = new TextField("Pseudo", null, 12, TextField.ANY);
    exportServerField = new TextField("Export URL", null, 50, TextField.URL);
    String[] stringArray = { "Petit", "Moyen", "Large" };

    // create the list of choice groups.
    choixTaille = new ChoiceGroup("Police", Choice.EXCLUSIVE, stringArray, null);

    optionForm = new Form("Paramtrage");
    optionForm.append(fullDistanceField);
    optionForm.append(lapDistanceField);
    optionForm.append(choixTaille);
    optionForm.append(pseudoField);
    optionForm.append(exportServerField);
    optionForm.addCommand(okCommand);
    optionForm.setCommandListener(this);

  }

  // appel  la fermeture de l'application
  protected void destroyApp(boolean unconditional)
  {
    _timer.cancel(); // arret du timer
  }

  // appel au dmarrage de l'application
  protected void startApp() {

    if (!firstStartDone) {
      try {
        // Le SplashScreen CourirLeMonde (BUG: ne pas afficher quand c'est un resume de l'appli)
        _display.setCurrent(new SplashScreen("/ClmChronoSplash.png",_display,_moteur));
        firstStartDone = true;    
      } catch (IOException ioex) {
        // Sinon, direct l'cran principal de 'appli.
        _display.setCurrent(_moteur); // _moteur hrite de Canvas      
      }
    } else {
      _display.setCurrent(_moteur); // _moteur hrite de Canvas       
    }

    try {
    _timer.schedule(_clmTimer, 1, 10); // toutes les 20 ms, _clmTimer est appel
    } catch (IllegalStateException ex) {
      // do nothing : if already sheduled : good (resuming the app)
    }

  }

  // appel lors de la mise en pause de l'application
  protected void pauseApp() {}

    public void commandAction(Command command, Displayable displayable) {
    
        long now = new Date().getTime();
        long diff = now - _moteur._stopTimeStamp;
          
        if(command == startCommand) {
	        _moteur.removeCommand(startCommand);
          _moteur.addCommand(stopCommand);
	        _moteur.removeCommand(optionCommand);
          _moteur.addCommand(lapCommand);
          _moteur.started = true;
          _moteur._startTimeStamp += diff;
        } else if(command == stopCommand) {
	        _moteur.stopChrono(now);
        } else if(command == resetCommand) {
          _moteur.init();
          _moteur.initChronoCommand();
        } else if(command == exportCommand) {
          displayExportForm();
        } else if(command == sendCommand) {
          export();
        } else if(command == backFromExportCommand) {
          _display.setCurrent(_moteur);
        } else if(command == lapCommand) {
	        _moteur.handleCommand(Canvas.KEY_POUND);
        } else if(command == optionCommand) {
          _moteur.readRecords();
	        _display.setCurrent(optionForm);
          fullDistanceField.setString("" + _moteur.getFullDistance());
          lapDistanceField.setString("" + _moteur.getLapDistance());
          choixTaille.setSelectedIndex(_moteur.getChoixPolice(),true);
          String exportServer = _moteur.getExportURL();
          if (exportServer==null || exportServer.equals("")) {
            exportServer = _moteur.defaultExportURL;
          }
          exportServerField.setString("" + exportServer);
          pseudoField.setString("" + _moteur.getPseudo());
          _display.setCurrent(optionForm);
        } else if(command == okCommand) {
          try {
            int i = Integer.parseInt(fullDistanceField.getString());
            _moteur.setFullDistance(i);
            int j = Integer.parseInt(lapDistanceField.getString());
            _moteur.setLapDistance(j);
            _display.setCurrent(_moteur);
            _moteur.setPseudo(pseudoField.getString());
            String exportServer = exportServerField.getString();
            if (exportServer==null || exportServer.equals("")) {
              exportServer = _moteur.defaultExportURL;
            }
            _moteur.setExportURL(exportServer);
            _moteur.setChoixPolice(choixTaille.getSelectedIndex()); 
            _moteur.saveConfig();
          } catch (NumberFormatException ex) {
            // On ne fait rien, en tout cas on reste sur l'ecran de paramtrage !!
          }
        }


    }

  public void displayExportForm() {
    Form f = new Form("ClmChrono - Export");
    // A field for naming the data to be exported
    titreField = new TextField("Titre:", "", 30, TextField.ANY);
    f.append(titreField);
    statusStringItem = new StringItem("Status:", "...");
    f.append(statusStringItem);
    f.addCommand(sendCommand);
    f.addCommand(backFromExportCommand);
    f.setCommandListener(this);
    _display.setCurrent(f); 
  }

  public void export() {

    String userAgent = getAppProperty("MIDlet-Name")+"/"+
                       getAppProperty("MIDlet-Version")+" ("+
                      System.getProperty("microedition.platform")+")";

    try {
      ClmNetwork net = new ClmNetwork(_moteur,userAgent,statusStringItem,titreField.getString());
    } catch (ConnectionNotFoundException cnfe) {
      statusStringItem.setText("ConnectionNotFound");
    } catch (IOException ex) {
      statusStringItem.setText("IOException");
    }

  }
    

}


